@@ -5,7 +5,7 @@ simulations <- function(tree.tls, tree.ds = NULL, tree.field,
                                                      k.max = 50, BAF.max = 4),
                         scan.approach = "single",
                         var.metr = list(tls = NULL, field = NULL),
-                        dbh.min = 4, h.min = 1.3,
+                        v.calc = "parab", dbh.min = 4, h.min = 1.3,
                         max.dist = Inf, dir.data = NULL, save.result = TRUE,
                         dir.result = NULL) {
 
@@ -15,7 +15,7 @@ simulations <- function(tree.tls, tree.ds = NULL, tree.field,
                    tree.field = tree.field, plot.design = plot.design,
                    plot.parameters = plot.parameters,
                    scan.approach = scan.approach, var.metr = var.metr,
-                   dbh.min = dbh.min, h.min = h.min,
+                   v.calc = v.calc, dbh.min = dbh.min, h.min = h.min,
                    max.dist = max.dist, dir.data = dir.data,
                    save.result = save.result, dir.result = dir.result)
   return(sim)
